@@ -21,17 +21,14 @@ class LocalShell(object):
         open(remote_path, "w").write(contents)
 
     def spawn(self, *args, **kwargs):
-        subprocess.Popen(**self._subprocess_args(*args, **kwargs))
+        process = subprocess.Popen(**self._subprocess_args(*args, **kwargs))
+        return LocalProcess(process)
         
     def run(self, *args, **kwargs):
         allow_error = kwargs.pop("allow_error", False)
-        process = subprocess.Popen(**self._subprocess_args(*args, **kwargs))
-        stdout, stderr = process.communicate()
-        return_code = process.poll()
+        process = self.spawn(*args, **kwargs)
         return spur.results.result(
-            return_code,
-            stdout,
-            stderr,
+            process,
             allow_error=allow_error
         )
         
@@ -47,7 +44,8 @@ class LocalShell(object):
             "args": command,
             "cwd": cwd,
             "stdout": subprocess.PIPE,
-            "stderr": subprocess.PIPE
+            "stderr": subprocess.PIPE,
+            "stdin": subprocess.PIPE
         }
         if update_env is not None:
             new_env = os.environ.copy()
@@ -56,3 +54,22 @@ class LocalShell(object):
         if new_process_group:
             kwargs["preexec_fn"] = os.setpgrp
         return kwargs
+
+class LocalProcess(object):
+    def __init__(self, subprocess):
+        self._subprocess = subprocess
+        
+    def is_running(self):
+        return self._subprocess.poll() is None
+        
+    def stdin_write(self, value):
+        self._subprocess.stdin.write(value)
+        
+    def wait_for_result(self):
+        stdout, stderr = self._subprocess.communicate()
+        return_code = self._subprocess.poll()
+        return spur.results.ExecutionResult(
+            return_code,
+            stdout,
+            stderr
+        )
